==> lamarc-2.1.src.tar.gz: lamarc-2.1/src/tree/collmanager.cpp <==
//$Id: collmanager.cpp,v 1.14 2006/04/01 00:34:10 lpsmith Exp $
/* 
 Copyright 2003 Peter Beerli, Mary Kuhner, Jon Yamato and Joseph Felsenstein

 This software is distributed free of charge for non-commercial use
 and is copyrighted.  Of course, we do not guarantee that the software
 works, and are not responsible for any damage you may cause or have.
*/

#include "collmanager.h"
#include "registry.h"
#include "stringx.h"
#include "newick.h"

using namespace std;

//____________________________________________________________________

CollectionManager::CollectionManager(long numreg, long numrep)
 : m_region(0),
   m_replicate(0)
{
  //This used to be the EstablishSamplingStrategy function, but it works
  // semantically better here --LS

  // This function implements a very simple rule for which aspects
  // of the chain to sample.  It will need to get more complex 
  // eventually.

  ChainParameters& chainparam = registry.GetChainParameters();

  bool bayesian = chainparam.IsBayesian();

  if (bayesian) {
    m_sample_params = true;
#ifdef STATIONARIES
    m_sample_trees = true;
#else
    m_sample_trees = false;
#endif
  } else {
    m_sample_params = false;
    m_sample_trees = true;
  }

  m_sample_maps = false;    //We'll set this to true in StartChain if needed.
  m_sample_sticks = false;  // since we don't know how yet!

  // set up internal storage
  long reg, rep;
  vector<TreeCollector*> temptree;
  vector<ParamCollector*> tempparam;
  vector<StickCollector*> tempstick;
  vector<MapCollector*> tempmap;
  for (reg = 0; reg < numreg; ++reg) {
    for (rep = 0; rep < numrep; ++rep) { 
      temptree.push_back(new TreeCollector);
      tempparam.push_back(new ParamCollector);
      tempstick.push_back(new StickCollector);
      tempmap.push_back(new MapCollector(reg));
      //Note:  These usually get destroyed by StartChain(), but not in the
      // case of reading in from a summary file (ChainManager::ReadInSumFile)
    }
    m_treecoll.push_back(temptree);
    m_paramcoll.push_back(tempparam);
    m_stickcoll.push_back(tempstick);
    m_mapcoll.push_back(tempmap);
    temptree.clear();
    tempparam.clear();
    tempstick.clear();
    tempmap.clear();
  }

} /* CollectionManager ctor */

//____________________________________________________________________

CollectionManager::~CollectionManager()
{
  // Clean up the Collectors
  unsigned long reg, rep;
  for (reg = 0; reg < m_treecoll.size(); ++reg) {
    for (rep = 0; rep < m_treecoll[reg].size(); ++rep) {
      delete m_treecoll[reg][rep];
      delete m_paramcoll[reg][rep];
      delete m_stickcoll[reg][rep];
      delete m_mapcoll[reg][rep];
    }
  }
} /* CollectionManager dtor */

//____________________________________________________________________

void CollectionManager::Collect(ChainState& chstate)
{
  if (m_sample_trees) m_treecoll[m_region][m_replicate]->Score(chstate);
  if (m_sample_params) m_paramcoll[m_region][m_replicate]->Score(chstate);
  if (m_sample_sticks) m_stickcoll[m_region][m_replicate]->Score(chstate);
  if (m_sample_maps) m_mapcoll[m_region][m_replicate]->Score(chstate);
  chstate.ParametersSampled();//Might as well make it consistent.
  WriteTraceFile(chstate);
  WriteNewickTreeFile(chstate);
} /* Collect */

void CollectionManager::WriteTraceFile(ChainState& chstate)
{
  UserParameters& userparams = registry.GetUserParameters();
  if (userparams.GetWriteTraceFiles() &&
      !registry.GetChainParameters().IsBayesian()) {
    string tracefilename = userparams.GetCurrentTraceFileName();
    long step = userparams.GetNextStep();
    double datalike = chstate.GetTree()->GetDLValue();
    ofstream of;
    of.open(tracefilename.c_str(),ios::out | ios::app);
    of << ToString(step) << "\t" << ToString(datalike) << endl;
    of.close();
  }
}

void CollectionManager::WriteNewickTreeFile(ChainState& chstate)
{
  UserParameters& userparams = registry.GetUserParameters();
  if (userparams.GetWriteNewickTreeFiles()) {
    const Tree* tree = chstate.GetTree();
    double dl = tree->GetDLValue();
    double bestlike = userparams.GetCurrentBestLike();
    string newickfilename = userparams.GetCurrentNewickTreeFileName();
    if (dl > bestlike) {
      userparams.SetCurrentBestLike(dl);
      NewickConverter nc;
      string newick = nc.LamarcTreeToNewickString(*tree);
      ofstream of;
      of.open(newickfilename.c_str(),ios::out | ios::trunc);
      of << dl << endl;
      of << newick << endl;
      of.close();
      userparams.AddNewickTreeFileName(newickfilename);
    }
  }
}
//____________________________________________________________________

void CollectionManager::CorrectForFatalAttraction(long region)
{
  if (m_sample_trees) 
    m_treecoll[m_region][m_replicate]->CorrectForFatalAttraction(region);
  if (m_sample_params) 
    m_paramcoll[m_region][m_replicate]->CorrectForFatalAttraction(region);
  if (m_sample_sticks) 
    m_stickcoll[m_region][m_replicate]->CorrectForFatalAttraction(region);
  if (m_sample_maps) 
    m_mapcoll[m_region][m_replicate]->CorrectForFatalAttraction(region);
} /* CorrectForFatalAttraction */

//____________________________________________________________________

void CollectionManager::StartChain(long reg, long rep, bool lastchain)
{
  //LS Note:  If you add code in this function, make sure that this doesn't
  // break reading in partially-written summary files.
  // See ChainManager::ReadInSumFile and ChainManager::ReadInRecover.

  // this is another chain for the same region and replicate.
  // We store only one chain per region/replicate pair, so throw
  // away the old one and store this one.
  delete m_treecoll[reg][rep];
  delete m_paramcoll[reg][rep];
  delete m_stickcoll[reg][rep];
  delete m_mapcoll[reg][rep];

  m_region = reg;
  m_replicate = rep;

  m_treecoll[reg][rep]  = new TreeCollector;
  m_paramcoll[reg][rep] = new ParamCollector;
  m_stickcoll[reg][rep] = new StickCollector;
  m_mapcoll[reg][rep] = new MapCollector(reg);

  m_sample_maps = m_mapcoll[reg][rep]->DoWeCollectFor(lastchain);
} /* StartChain */

//____________________________________________________________________

void CollectionManager::WriteThisChainsCollections(std::ofstream* out) const
{
  if (m_sample_trees) 
    m_treecoll[m_region][m_replicate]->WriteCollectionWhenScore(out);
  if (m_sample_params) 
    m_paramcoll[m_region][m_replicate]->WriteCollectionWhenScore(out);
  if (m_sample_sticks)
    m_stickcoll[m_region][m_replicate]->WriteCollectionWhenScore(out);
  if (m_sample_maps)
    m_mapcoll[m_region][m_replicate]->WriteCollectionWhenScore(out);
}

void CollectionManager::WriteLastSummaries() const
{
  if (m_sample_trees) 
    m_treecoll[m_region][m_replicate]->WriteLastSummary();
  if (m_sample_params) 
    m_paramcoll[m_region][m_replicate]->WriteLastSummary();
  if (m_sample_sticks) 
    m_stickcoll[m_region][m_replicate]->WriteLastSummary();
  if (m_sample_maps) 
    m_mapcoll[m_region][m_replicate]->WriteLastSummary();
}

void CollectionManager::WriteAllSummaries(long reg, long rep,
                                          std::ofstream& out) const
{
  if (m_sample_trees) 
    m_treecoll[reg][rep]->WriteAllSummaries(out);
  if (m_sample_params) 
    m_paramcoll[reg][rep]->WriteAllSummaries(out);
  if (m_sample_sticks) 
    m_stickcoll[reg][rep]->WriteAllSummaries(out);
  if (m_sample_maps) 
    m_mapcoll[reg][rep]->WriteAllSummaries(out);
}

//____________________________________________________________________

TreeCollector* CollectionManager::GetTreeColl(long region, long rep) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_treecoll.size()));
  assert(rep < static_cast<long>(m_treecoll[region].size()));
  assert(m_sample_trees);

  return m_treecoll[region][rep];
} /* GetTreeColl */

//____________________________________________________________________

vector<TreeCollector*> CollectionManager::GetTreeColl(long region) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_treecoll.size()));
  assert(m_sample_trees);
  
  return m_treecoll[region];
} /* GetTreeColl */

//____________________________________________________________________

vector<vector<TreeCollector*> > CollectionManager::GetTreeColl() const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(m_sample_trees);
  
  return m_treecoll;
} /* GetTreeColl */
//____________________________________________________________________

ParamCollector* CollectionManager::GetParamColl(long region, long rep) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_paramcoll.size()));
  assert(rep < static_cast<long>(m_paramcoll[region].size()));
  assert(m_sample_params);

  return m_paramcoll[region][rep];
} /* GetParamColl */

//____________________________________________________________________

vector<ParamCollector*> CollectionManager::GetParamColl(long region) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_paramcoll.size()));
  assert(m_sample_params);

  return m_paramcoll[region];
} /* GetParamColl */

//____________________________________________________________________

vector<vector<ParamCollector*> > CollectionManager::GetParamColl() const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(m_sample_params);

  return m_paramcoll;

} /* GetParamColl */

//____________________________________________________________________

StickCollector* CollectionManager::GetStickColl(long region, long rep) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_stickcoll.size()));
  assert(rep < static_cast<long>(m_stickcoll[region].size()));
  assert(m_sample_sticks);

  return m_stickcoll[region][rep];
} /* GetStickColl */

//____________________________________________________________________

vector<StickCollector*> CollectionManager::GetStickColl(long region) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_stickcoll.size()));
  assert(m_sample_sticks);

  return m_stickcoll[region];
} /* GetStickColl */

//____________________________________________________________________

vector<vector<StickCollector*> > CollectionManager::GetStickColl() const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(m_sample_sticks);

  return m_stickcoll;
} /* GetStickColl */

//____________________________________________________________________

MapCollector* CollectionManager::GetMapColl(long region, long rep) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_mapcoll.size()));
  assert(rep < static_cast<long>(m_mapcoll[region].size()));
  assert(m_mapcoll[region][rep]->DoWeCollectFor(true));

  return m_mapcoll[region][rep];
} /* GetMapColl */

//____________________________________________________________________

vector<MapCollector*> CollectionManager::GetMapColl(long region) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_mapcoll.size()));
  assert(m_mapcoll[region][0]->DoWeCollectFor(true));

  return m_mapcoll[region];
} /* GetMapColl */

//____________________________________________________________________

vector<vector<MapCollector*> > CollectionManager::GetMapColl() const
{
  // We only collect for some regions, and can't tell (here) which ones.
  // assert(m_sample_maps);

  return m_mapcoll;
} /* GetMapColl */

//____________________________________________________________________

void CollectionManager::UpdateArrangers(ArrangerVec& arrangers, ChainState&
  chstate) const
{
// Currently, only parameter changes trigger arranger updates.
// This can easily be changed here.

m_paramcoll[m_region][m_replicate]->UpdateArrangers(arrangers, chstate);

} /* UpdateArrangers */

//____________________________________________________________________

==> lamarc-2.1.2b.src.tar.gz: lamarc-2.1.2b/src/tree/collmanager.cpp <==
//$Id: collmanager.cpp,v 1.16 2007/08/03 20:46:31 lpsmith Exp $
/* 
 Copyright 2003 Peter Beerli, Mary Kuhner, Jon Yamato and Joseph Felsenstein

 This software is distributed free of charge for non-commercial use
 and is copyrighted.  Of course, we do not guarantee that the software
 works, and are not responsible for any damage you may cause or have.
*/

#include "collmanager.h"
#include "registry.h"
#include "stringx.h"
#include "newick.h"
#include "parameter.h"

using namespace std;

//____________________________________________________________________

CollectionManager::CollectionManager(long numreg, long numrep)
 : m_region(0),
   m_replicate(0)
{
  //This used to be the EstablishSamplingStrategy function, but it works
  // semantically better here --LS

  // This function implements a very simple rule for which aspects
  // of the chain to sample.  It will need to get more complex 
  // eventually.

  ChainParameters& chainparam = registry.GetChainParameters();

  bool bayesian = chainparam.IsBayesian();

  if (bayesian) {
    m_sample_params = true;
#ifdef STATIONARIES
    m_sample_trees = true;
#else
    m_sample_trees = false;
#endif
  } else {
    m_sample_params = false;
    m_sample_trees = true;
  }

  m_sample_maps = false;    //We'll set this to true in StartChain if needed.

  // set up internal storage
  long reg, rep;
  vector<TreeCollector*> temptree;
  vector<ParamCollector*> tempparam;
  vector<MapCollector*> tempmap;
  for (reg = 0; reg < numreg; ++reg) {
    for (rep = 0; rep < numrep; ++rep) { 
      temptree.push_back(new TreeCollector);
      tempparam.push_back(new ParamCollector);
      tempmap.push_back(new MapCollector(reg));
      //Note:  These usually get destroyed by StartChain(), but not in the
      // case of reading in from a summary file (ChainManager::ReadInSumFile)
    }
    m_treecoll.push_back(temptree);
    m_paramcoll.push_back(tempparam);
    m_mapcoll.push_back(tempmap);
    temptree.clear();
    tempparam.clear();
    tempmap.clear();
  }

} /* CollectionManager ctor */

//____________________________________________________________________

CollectionManager::~CollectionManager()
{
  // Clean up the Collectors
  unsigned long reg, rep;
  for (reg = 0; reg < m_treecoll.size(); ++reg) {
    for (rep = 0; rep < m_treecoll[reg].size(); ++rep) {
      delete m_treecoll[reg][rep];
      delete m_paramcoll[reg][rep];
      delete m_mapcoll[reg][rep];
    }
  }
} /* CollectionManager dtor */

//____________________________________________________________________

void CollectionManager::Collect(ChainState& chstate)
{
  // stick sampling is automatically done as part of tree sampling
  if (m_sample_trees) m_treecoll[m_region][m_replicate]->Score(chstate);
  if (m_sample_params) m_paramcoll[m_region][m_replicate]->Score(chstate);
  if (m_sample_maps) m_mapcoll[m_region][m_replicate]->Score(chstate);
  chstate.ParametersSampled();//Might as well make it consistent.
  WriteTraceFile(chstate);
  WriteNewickTreeFile(chstate);
} /* Collect */

void CollectionManager::WriteTraceFile(ChainState& chstate)
{
  //Note:  The column headers for this file are written in
  // UserParameters::UpdateFileNamesAndSteps(...) in userparam.cpp
  UserParameters& userparams = registry.GetUserParameters();
  if (userparams.GetWriteTraceFiles()) {
    string tracefilename = userparams.GetCurrentTraceFileName();
    DoubleVec1d values;
    values.push_back(userparams.GetNextStep());
    values.push_back(chstate.GetTree()->GetDLValue());
    if (registry.GetChainParameters().IsBayesian()) {
      DoubleVec1d params = m_paramcoll[m_region][m_replicate]->GetLastParameterVec();
      const ParamVector pvec(true);
      assert(pvec.size() == params.size());
      for (size_t p=0; p<pvec.size(); p++) {
        if (pvec[p].IsValid()) {
          values.push_back(params[p]);
        }
      }
    }
    ofstream of;
    of.open(tracefilename.c_str(),ios::out | ios::app);
    of << ToString(values[0]);
    for (size_t i=1; i<values.size(); i++) {
      of << "\t" << ToString(values[i]);
    }
    of << endl;
    of.close();
  }
}

void CollectionManager::WriteNewickTreeFile(ChainState& chstate)
{
  UserParameters& userparams = registry.GetUserParameters();
  if (userparams.GetWriteNewickTreeFiles()) {
    const Tree* tree = chstate.GetTree();
    double dl = tree->GetDLValue();
    double bestlike = userparams.GetCurrentBestLike();
    string newickfilename = userparams.GetCurrentNewickTreeFileName();
    if (dl > bestlike) {
      userparams.SetCurrentBestLike(dl);
      NewickConverter nc;
      string newick = nc.LamarcTreeToNewickString(*tree);
      ofstream of;
      of.open(newickfilename.c_str(),ios::out | ios::trunc);
      of << dl << endl;
      of << newick << endl;
      of.close();
      userparams.AddNewickTreeFileName(newickfilename);
    }
  }
}
//____________________________________________________________________

void CollectionManager::CorrectForFatalAttraction(long region)
{
  if (m_sample_trees) 
    m_treecoll[m_region][m_replicate]->CorrectForFatalAttraction(region);
  if (m_sample_params) 
    m_paramcoll[m_region][m_replicate]->CorrectForFatalAttraction(region);
  if (m_sample_maps) 
    m_mapcoll[m_region][m_replicate]->CorrectForFatalAttraction(region);
} /* CorrectForFatalAttraction */

//____________________________________________________________________

void CollectionManager::StartChain(long reg, long rep, bool lastchain)
{
  //LS Note:  If you add code in this function, make sure that this doesn't
  // break reading in partially-written summary files.
  // See ChainManager::ReadInSumFile and ChainManager::ReadInRecover.

  // this is another chain for the same region and replicate.
  // We store only one chain per region/replicate pair, so throw
  // away the old one and store this one.
  delete m_treecoll[reg][rep];
  delete m_paramcoll[reg][rep];
  delete m_mapcoll[reg][rep];

  m_region = reg;
  m_replicate = rep;

  m_treecoll[reg][rep]  = new TreeCollector;
  m_paramcoll[reg][rep] = new ParamCollector;
  m_mapcoll[reg][rep] = new MapCollector(reg);

  m_sample_maps = m_mapcoll[reg][rep]->DoWeCollectFor(lastchain);
} /* StartChain */

//____________________________________________________________________

void CollectionManager::WriteThisChainsCollections(std::ofstream* out) const
{
  if (m_sample_trees) 
    m_treecoll[m_region][m_replicate]->WriteCollectionWhenScore(out);
  if (m_sample_params) 
    m_paramcoll[m_region][m_replicate]->WriteCollectionWhenScore(out);
  if (m_sample_maps)
    m_mapcoll[m_region][m_replicate]->WriteCollectionWhenScore(out);
}

void CollectionManager::WriteLastSummaries() const
{
  if (m_sample_trees) 
    m_treecoll[m_region][m_replicate]->WriteLastSummary();
  if (m_sample_params) 
    m_paramcoll[m_region][m_replicate]->WriteLastSummary();
  if (m_sample_maps) 
    m_mapcoll[m_region][m_replicate]->WriteLastSummary();
}

void CollectionManager::WriteAllSummaries(long reg, long rep,
                                          std::ofstream& out) const
{
  if (m_sample_trees) 
    m_treecoll[reg][rep]->WriteAllSummaries(out);
  if (m_sample_params) 
    m_paramcoll[reg][rep]->WriteAllSummaries(out);
  if (m_sample_maps) 
    m_mapcoll[reg][rep]->WriteAllSummaries(out);
}

//____________________________________________________________________

TreeCollector* CollectionManager::GetTreeColl(long region, long rep) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_treecoll.size()));
  assert(rep < static_cast<long>(m_treecoll[region].size()));
  assert(m_sample_trees);

  return m_treecoll[region][rep];
} /* GetTreeColl */

//____________________________________________________________________

vector<TreeCollector*> CollectionManager::GetTreeColl(long region) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_treecoll.size()));
  assert(m_sample_trees);
  
  return m_treecoll[region];
} /* GetTreeColl */

//____________________________________________________________________

vector<vector<TreeCollector*> > CollectionManager::GetTreeColl() const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(m_sample_trees);
  
  return m_treecoll;
} /* GetTreeColl */
//____________________________________________________________________

ParamCollector* CollectionManager::GetParamColl(long region, long rep) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_paramcoll.size()));
  assert(rep < static_cast<long>(m_paramcoll[region].size()));
  assert(m_sample_params);

  return m_paramcoll[region][rep];
} /* GetParamColl */

//____________________________________________________________________

vector<ParamCollector*> CollectionManager::GetParamColl(long region) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_paramcoll.size()));
  assert(m_sample_params);

  return m_paramcoll[region];
} /* GetParamColl */

//____________________________________________________________________

vector<vector<ParamCollector*> > CollectionManager::GetParamColl() const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(m_sample_params);

  return m_paramcoll;

} /* GetParamColl */

//____________________________________________________________________

MapCollector* CollectionManager::GetMapColl(long region, long rep) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_mapcoll.size()));
  assert(rep < static_cast<long>(m_mapcoll[region].size()));
  assert(m_mapcoll[region][rep]->DoWeCollectFor(true));

  return m_mapcoll[region][rep];
} /* GetMapColl */

//____________________________________________________________________

vector<MapCollector*> CollectionManager::GetMapColl(long region) const
{
  // are we trying to retrieve a chain we didn't expect?  That's bad.
  assert(region < static_cast<long>(m_mapcoll.size()));
  assert(m_mapcoll[region][0]->DoWeCollectFor(true));

  return m_mapcoll[region];
} /* GetMapColl */

//____________________________________________________________________

vector<vector<MapCollector*> > CollectionManager::GetMapColl() const
{
  // We only collect for some regions, and can't tell (here) which ones.
  // assert(m_sample_maps);

  return m_mapcoll;
} /* GetMapColl */

//____________________________________________________________________

void CollectionManager::UpdateArrangers(ArrangerVec& arrangers, ChainState&
  chstate) const
{
// Currently, only parameter changes trigger arranger updates.
// This can easily be changed here.

m_paramcoll[m_region][m_replicate]->UpdateArrangers(arrangers, chstate);

} /* UpdateArrangers */

//____________________________________________________________________
